@@ -5,11 +5,11 @@ use warnings;
 package Session::Storage::Secure;
 # ABSTRACT: Encrypted, expiring, compressed, serialized session data with integrity
 
-our $VERSION = '0.011';
+our $VERSION = '1.000';
 
-use Carp                    (qw/croak/);
-use Crypt::CBC              ();
-use Crypt::Rijndael         ();
+use Carp (qw/croak/);
+use Crypt::CBC 3.01 ();
+use Crypt::Rijndael ();
 use Crypt::URandom          (qw/urandom/);
 use Digest::SHA             (qw/hmac_sha256/);
 use Math::Random::ISAAC::XS ();
@@ -134,6 +134,20 @@ has transport_decoder => (
     default => sub { \&MIME::Base64::decode_base64url },
 );
 
+#pod =attr protocol_version
+#pod
+#pod An integer representing the protocol used by C<Session::Storage::Secure>.
+#pod Protocol 1 was the initial version, which used a now-deprecated mode of
+#pod L<Crypt::CBC>.  Protocol 2 is the current default.
+#pod
+#pod =cut
+
+has protocol_version => (
+    is      => 'ro',
+    isa     => Num,
+    default => 2,
+);
+
 has _encoder => (
     is      => 'lazy',
     isa     => InstanceOf ['Sereal::Encoder'],
@@ -167,6 +181,41 @@ sub _build__rng {
     return Math::Random::ISAAC::XS->new( map { unpack( "N", urandom(4) ) } 1 .. 256 );
 }
 
+sub BUILD {
+    my ($self) = @_;
+    $self->_check_version_for( encoding => $self->protocol_version );
+}
+
+sub _check_version_for {
+    my ( $self, $action, $pv ) = @_;
+    if ( $pv < 1 || $pv > 2 ) {
+        croak "Invalid protocol version for $action: $pv";
+    }
+}
+
+sub _get_cbc {
+    my ( $self, $pv, $key, $salt ) = @_;
+
+    my $cbc_opts = {
+        -key    => $key,
+        -cipher => 'Rijndael',
+    };
+
+    if ( $pv == 1 ) {
+        $cbc_opts->{-pbkdf}       = 'opensslv1';
+        $cbc_opts->{-nodeprecate} = 1;
+    }
+    else {
+        $cbc_opts->{-pbkdf}       = 'none';
+        $cbc_opts->{-keysize}     = 32;
+        $cbc_opts->{-header}      = 'none';
+        my $cipher = Crypt::Rijndael->new($key);
+        $cbc_opts->{-iv} = substr( $cipher->encrypt($salt), 0, 16 );
+    }
+
+    return Crypt::CBC->new(%$cbc_opts);
+}
+
 #pod =method encode
 #pod
 #pod   my $string = $store->encode( $data, $expires );
@@ -204,10 +253,20 @@ sub encode {
     }
 
     # Random salt used to derive unique encryption/MAC key for each cookie
-    my $salt = $self->_irand;
+    my $salt;
+    if ( $self->protocol_version == 1 ) {
+        # numeric salt
+        $salt = $self->_irand;
+    }
+    else {
+        # binary salt
+        $salt = pack( "N*", map { $self->_irand } 1 .. 8 );
+    }
+
     my $key = hmac_sha256( $salt, $self->secret_key );
 
-    my $cbc = Crypt::CBC->new( -key => $key, -cipher => 'Rijndael' );
+    my $cbc = $self->_get_cbc( $self->protocol_version, $key, $salt );
+
     my ( $ciphertext, $mac );
     eval {
         $ciphertext = $self->transport_encoder->( $cbc->encrypt( $self->_freeze($data) ) );
@@ -215,7 +274,15 @@ sub encode {
     };
     croak "Encoding error: $@" if $@;
 
-    return join( $sep, $salt, $expires, $ciphertext, $mac );
+    my $output;
+    if ( $self->protocol_version == 1 ) {
+        $output = join( $sep, $salt, $expires, $ciphertext, $mac );
+    }
+    else {
+        $salt   = $self->transport_encoder->($salt);
+        $output = join( $sep, $salt, $expires, $ciphertext, $mac, $self->protocol_version );
+    }
+    return $output;
 }
 
 #pod =method decode
@@ -237,9 +304,19 @@ sub decode {
 
     # Having a string implies at least salt; expires is optional; rest required
     my $sep = $self->separator;
-    my ( $salt, $expires, $ciphertext, $mac ) = split qr/\Q$sep\E/, $string;
+    my ( $salt, $expires, $ciphertext, $mac, $version ) = split qr/\Q$sep\E/, $string;
     return unless defined($ciphertext) && length($ciphertext);
     return unless defined($mac)        && length($mac);
+    $version = 1 unless defined $version;
+    $self->_check_version_for( decoding => $version );
+
+    if ( $version == 1 ) {
+        # $salt is a decimal
+    }
+    else {
+        # Decode salt to binary
+        $salt = $self->transport_decoder->($salt);
+    }
 
     # Try to decode against all known secret keys
     my @secrets = ( $self->secret_key, @{ $self->old_secrets || [] } );
@@ -265,7 +342,8 @@ sub decode {
     return if length($expires) && $expires < time;
 
     # Decrypt and deserialize the data
-    my $cbc = Crypt::CBC->new( -key => $key, -cipher => 'Rijndael' );
+    my $cbc = $self->_get_cbc( $version, $key, $salt );
+
     my $data;
     eval {
         $self->_thaw( $cbc->decrypt( $self->transport_decoder->($ciphertext) ), $data );
@@ -292,7 +370,7 @@ Session::Storage::Secure - Encrypted, expiring, compressed, serialized session d
 
 =head1 VERSION
 
-version 0.011
+version 1.000
 
 =head1 SYNOPSIS
 
@@ -433,6 +511,12 @@ A code reference to extract binary data (the encrypted data and the
 MAC) from a transport-safe form.  It must be the complement to C<encode>.
 Defaults to L<MIME::Base64::decode_base64url|MIME::Base64>.
 
+=head2 protocol_version
+
+An integer representing the protocol used by C<Session::Storage::Secure>.
+Protocol 1 was the initial version, which used a now-deprecated mode of
+L<Crypt::CBC>.  Protocol 2 is the current default.
+
 =head1 METHODS
 
 =head2 encode
@@ -466,7 +550,7 @@ the past, the method returns undef or an empty list (depending on context).
 
 An exception is thrown on any errors.
 
-=for Pod::Coverage has_default_duration
+=for Pod::Coverage has_default_duration BUILD
 
 =head1 LIMITATIONS
 
@@ -650,35 +734,26 @@ L<Data::Serializer>
 
 =back
 
-=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
-
-=head1 SUPPORT
-
-=head2 Bugs / Feature Requests
-
-Please report any bugs or feature requests through the issue tracker
-at L<https://github.com/dagolden/Session-Storage-Secure/issues>.
-You will be notified automatically of any progress on your issue.
-
-=head2 Source Code
-
-This is open source software.  The code repository is available for
-public review and contribution under the terms of the license.
-
-L<https://github.com/dagolden/Session-Storage-Secure>
-
-  git clone https://github.com/dagolden/Session-Storage-Secure.git
-
 =head1 AUTHOR
 
 David Golden <dagolden@cpan.org>
 
-=head1 CONTRIBUTOR
+=head1 CONTRIBUTORS
 
-=for stopwords Tom Hukins
+=for stopwords Petr Písař Tom Hukins
+
+=over 4
+
+=item *
+
+Petr Písař <ppisar@redhat.com>
+
+=item *
 
 Tom Hukins <tom@eborcom.com>
 
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is Copyright (c) 2013 by David Golden.
